package altbase;

use vars qw($db $dbUser $dbPass $dbHost);
require 'dbinfo';

use strict;

BEGIN {
    use Exporter   ();
    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

    # set the version for version checking
    $VERSION     = 1.00;

    @ISA         = qw(Exporter);
    @EXPORT      = qw(&new &init &parser &MakeDateTime &MakeDate &urlEscape &tagClean &tagConv &isInArray &specColorer &changeFormat &tagHardClean &messages &transPath &localeGrp &strBreak);
    %EXPORT_TAGS = ( );
    @EXPORT_OK   = qw($site $path @month $linkstyle $errCode $errStr $header $redirect $maxId $maxPage $dsn $dbUser $dbPass $TTL $repocopSupport);
}

our @EXPORT_OK;
# exported package globals go here
our @month	= () ;
our %totalGroups = ();
our %messages	= ();
our $linkstyle	= undef;
our $errCode	= 0;
our $errStr	= undef;
our $header	= "Content-type: text/html\n\n";
our $redirect	= "Status: 302 Moved\nLocation: ";
our $maxId	= 100000;
our $maxPage	= 1000;
our $site	= "http://alt5.linux.kiev.ua";
our $path	= "";
our $TTL	= 3600*2; # Time to user stay logged in
our $gitpath	= "http://git.altlinux.org";

our $dsn    = "DBI:mysql:$db:$dbHost";	# just $dsn, no reason to change value

our $repocopSupport=-e '/usr/bin/repocop-report-prometeus'; # disabled by default

my %HTML = ();

my $lang = 'ru';
#my $htf_root  = "/www/alt5.linux.kiev.ua/html/htf" ;
my $htf_root  = "/usr/share/prometeus/web/html/htf/";

my %modules = (
                "inform"   => "$htf_root",
                "maintain" => "$htf_root"
);

our @repos = ("Sisyphus","Daedalus","Branch5","Branch41","UP41","BP41","Branch4","UP4","BP4","Branch3","UP3","BP3","orphaned","obsolete");

our %repoinfo =
(
 "Sisyphus" =>
 {
  "rpm" => "Sisyphus/files/<arch>/RPMS/",
  "srpm" => "Sisyphus/files/SRPMS",
 },
 "Daedalus" =>
 {
  "rpm" => "Daedalus/<arch>/RPMS.daedalus/",
  "srpm" => "Daedalus/SRPMS.daedalus",
 },
 "Branch5" =>
 {
  "rpm" => "5.0/branch/files/<arch>/RPMS/",
  "srpm" => "5.0/branch/files/SRPMS",
 },
 "Branch41" =>
 {
  "rpm" => "4.1/branch/files/<arch>/RPMS/",
  "srpm" => "4.1/branch/files/SRPMS",
 },
 "UP41" =>
 {
  "rpm" => "updates/4.1/<arch>/RPMS.updates/",
  "srpm" => "updates/4.1/SRPMS.updates",
 },
 "BP41" =>
 {
  "rpm" => "backports/4.1/<arch>/RPMS.backports/",
  "srpm" => "backports/4.1/SRPMS.backports",
 },
 "Branch4" =>
 {
  "rpm" => "4.0/branch/files/<arch>/RPMS/",
  "srpm" => "4.0/branch/files/SRPMS",
 },
 "UP4" =>
 {
  "rpm" => "updates/4.0/<arch>/RPMS.updates/",
  "srpm" => "updates/4.0/SRPMS.updates",
 },
 "BP4" =>
 {
  "rpm" => "backports/4.0/<arch>/RPMS.backports/",
  "srpm" => "backports/4.0/SRPMS.backports",
 },
 "Branch3" =>
 {
  "rpm" => "3.0/files/<arch>/RPMS/",
  "srpm" => "3.0/files/SRPMS",
 },
 "UP3" =>
 {
  "rpm" => "updates/3.0/<arch>/RPMS.updates/",
  "srpm" => "updates/3.0/SRPMS.updates",
 },
 "BP3" =>
 {
  "rpm" => "backports/3.0/<arch>/RPMS.backports/",
  "srpm" => "backports/3.0/SRPMS.backports",
 },
 "orphaned" =>
 {
  "srpm" => "Sisyphus/orphaned",
 },
 "obsolete" =>
 {
  "srpm" => "Sisyphus/obsolete",
 },
);

END { }


# new class altbase
sub new {
	my $self = shift;
	$self = "altbase" if (! defined($self));
	my $class = ref($self) || $self;
	return bless {},$class;
}

# Check if defined class altbase
sub self_or_default {
	return @_ if (defined($_[0]) && (ref($_[0]) eq "altbase"));
	my $Q = new altbase;
	return $Q,@_;
}

# init HTML hash
sub init {
	my $name = "";
	%HTML = ();
	my ($class,%Param) = self_or_default(@_);
	$lang = $Param{'lang'} if ($Param{'lang'});
	if ( exists $modules{$Param{'module'}} ) {
	     my $moduledir = $modules{$Param{'module'}}."/$lang";
	     if ( -d $moduledir		&&
		opendir(DIR,$moduledir) ){
		my @filelist = grep { /\.htf$/ && -f $moduledir."/".$_ } readdir(DIR);
		closedir(DIR);
		foreach my $file (@filelist) {
			if ($file =~ /^(?:GLOBAL|$Param{'script'})_(\w+)\.htf$/ && open FILE, $moduledir."/".$file ){
				$name = $1;
				while (<FILE>) {
					$HTML{$name} .= $_;
				}
				chomp($HTML{$name});
				close FILE;
			}
		}

		if ( defined($HTML{"MONTH"}) ){
			@month = split(/\s/,$HTML{"MONTH"});
			delete $HTML{"MONTH"};
			$linkstyle = $HTML{"AHREF"};
			delete $HTML{"AHREF"};
		}
	    }
        if ( defined($HTML{"GROUPS"}) ){
            foreach my $line ( split(/\n/,$HTML{"GROUPS"}) ){
		my ($engrp,$tran) = split(/\:/, $line);
		$totalGroups{$engrp} = $tran;
	    }
            delete $HTML{"GROUPS"};
        }
	if ( defined($HTML{"MESSAGES"}) ){
            %messages = split(/\n/,$HTML{"MESSAGES"});
            delete $HTML{"MESSAGES"};
	}
	} else {
		$errCode = 1;
		$errStr = "not found HTML-codes for this module";
		return undef;
	}
	return 1;
}

# parse HTML-code & replace variables
sub parser {
	my ($class,%Param) = self_or_default(@_);
	my $text = $HTML{$Param{"HTML"}};
	my %vars = %{$Param{"vars"}};
	my %iflist = %{$Param{"iflist"}};

	$text =~ s/(?:\\\n)?\{(if\s|unless\s|if\s\!)(\w+)\}\n?(.+?)(?:(?:\\\n)?\{else\}\n?(.+?))?(?:\\\n)?\{endif\}\n?/($iflist{$2} != 0 ^ $1 eq "if ")?$4:$3/sge if (keys %iflist);
	$text =~ s/%(\w+)\s*(\w*)%/&transform($vars{$1},$2)/ge if (keys %vars);
	return $text;
}

sub transform {

	my $data = shift;
	my $tran = shift;
	
	if ($tran eq "h") {
		return tagClean($data);
	}
	if ($tran eq "H") {
		return tagHardClean($data);
	}
	if ($tran eq "t") {
		tagConv(\$data);
	}
	if ($tran eq "c") {
		$data = tagClean($data);
		$data =~ s/CVE-(\d{4})-(\d{4})/<a href="http:\/\/nvd.nist.gov\/nvd.cfm?cvename=CVE-$1-$2">CVE-$1-$2<\/a>/g; 
	}
	return $data;
}

sub urlEscape {
	my ($class,$text) = self_or_default(@_);
        $text =~ s/([^a-zA-Z0-9_\-. ])/uc sprintf("%%%02x",ord($1))/eg;
        $text =~ tr/ /+/;
        return $text;
}

sub tagClean {
	my ($class,$text) = self_or_default(@_);
	if (ref($text)){
		${$text} =~ s/&/&amp;/g;
		${$text} =~ s/>/&gt;/g;
		${$text} =~ s/</&lt;/g;
		${$text} =~ s/"/&quot;/g;
		${$text} =~ s/\n/<br \/>/g;
		${$text} =~ s/\r//g;
		return 0;
	} else {
		$text =~ s/&/&amp;/g;
		$text =~ s/>/&gt;/g;
		$text =~ s/</&lt;/g;
		$text =~ s/"/&quot;/g;
		$text =~ s/\n/<br \/>/g;
		$text =~ s/\r//g;
		return $text;
	}
}

sub tagHardClean {
	my ($class,$text) = self_or_default(@_);
	if (ref($text)){
		${$text} =~ s/&/&amp;/g;
		${$text} =~ s/>/&gt;/g;
		${$text} =~ s/</&lt;/g;
		${$text} =~ s/"/&quot;/g;
		${$text} =~ s/\n/ /g;
		${$text} =~ s/\r//g;
		return 0;
	} else {
		$text =~ s/&/&amp;/g;
		$text =~ s/>/&gt;/g;
		$text =~ s/</&lt;/g;
		$text =~ s/"/&quot;/g;
		$text =~ s/\n/ /g;
		$text =~ s/\r//g;
		return $text;
	}
}

sub changeFormat {
	my ($class,$text) = self_or_default(@_);
	if (ref($text)){
		${$text} =~ s/&/&amp;/g;
		${$text} =~ s/>/&gt;/g;
		${$text} =~ s/</&lt;/g;
		${$text} =~ s/"/&quot;/g;
		${$text} =~ s/^(\s*)\-\s/<li>/;
		${$text} =~ s/^(\s*)\-\s/<\/li><li>/g;
		${$text} =~ s/(\n\s*)\-\s/<\/li><li>/g;
		${$text} =~ s/\x20\x20/\x20&nbsp;/g;
		${$text} =~ s/\n/<br \/>/g;
		${$text} =~ s/\r//g;
		return 0;
	} else {
		$text =~ s/&/&amp;/g;
		$text =~ s/>/&gt;/g;
		$text =~ s/</&lt;/g;
		$text =~ s/"/&quot;/g;
		$text =~ s/^(\s*)\-\s/<li>/g;
		$text =~ s/^(\s*)\-\s/<\/li><li>/g;
		$text =~ s/(\n\s*)\-\s/<\/li><li>/g;
		$text =~ s/\x20\x20/\x20&nbsp;/g;
		$text =~ s/\n/<br \/>/g;
		$text =~ s/\r//g;
		return $text;
	}
}

sub specColorer{
	my ($class,$text) = self_or_default(@_);
	if (ref($text)){
		${$text}=~ s/&/&amp;/g;
		${$text}=~ s/>/&gt;/g;
		${$text}=~ s/</&lt;/g;
		${$text}=~ s/\"/&quot;/g;
		${$text}=~ s/\@/ at /g;
		${$text}=~ s/(Url\:\s+)(((http|ftp)\:\/\/[^\s\[\]]{1,100})([^\s\]\[]*))/"$1".&linkme($2,$3,$5)/gie;
		${$text}=~ s/\x20\x20/\x20&nbsp;/g;
		${$text}=~ s/^(\w+\:)/<b>$1<\/b>/g;
		${$text}=~ s/^(\s*\#.*)/<b class='comment'>$1<\/b>/g;
		${$text}=~ s/\n([\w\(\)\-\.]+\:)/\n<b>$1<\/b>/g;
		${$text}=~ s/\n(\s*\#.*)/\n<b class='comment'>$1<\/b>/g;
		${$text}=~ s/\n\%(description|prep|build|install|preun|pre|postun|post|triggerpostun|trigger|files|changelog|package)([\n|\s])/\n<b class='reserved'>\%$1<\/b>$2/g;
		${$text}=~ s/\n/<br \/>/g;
		${$text}=~ s/\r//g;
		return 0;
	} else {
		return undef;
	}
}

sub tagConv{
	my ($class,$text,$length) = self_or_default(@_);
    if (! defined($length) || $length < 5) {
        $length = 40;
    }

	if (ref($text)){
		${$text}=~ s/&/&amp;/g;
		${$text}=~ s/>/&gt;/g;
		${$text}=~ s/</&lt;/g;
		${$text}=~ s/\"/&quot;/g;
		${$text}=~ s/(((https?|ftp)\:\/\/[^\s\[\]]{1,$length})([^\s\]\[]*))/&linkme($1,$2,$4)/ge;
		${$text}=~ s/([\w\-\&\.]+\@[\w\-\&]+\.[\w\-\&\.]+)/<a href="mailto:$1" class="$linkstyle">$1<\/a>/g;
		${$text}=~ s/\[a\s(\<a[^\>]+?\>)[^\>]+?\>\]/$1/ig;
		${$text}=~ s/\[a\s([^\]]+?)\]/<a href="$1" class="$linkstyle">/ig;
		${$text}=~ s/\[\/a]/\<\/a\>/ig;
		${$text}=~ s/\[img\s+src\=\'?([^\']+?)\'?\]/\<IMG src=\"$1\"$2 vspace=\"4\" hspace=\"4\"\>/ig;
		${$text}=~ s/\[([\/]?)b\]/<$1b>/ig;
		${$text}=~ s/\[([\/]?)i\]/<$1i>/ig;
		${$text}=~ s/\[([\/]?)u\]/<$1u>/ig;
		${$text}=~ s/\[\&gt\;\&gt\;\]/<div align="right">/ig;
		${$text}=~ s/\[\&lt\;\&lt\;\]/<div align="left">/ig;
		${$text}=~ s/\[\&gt\;\&lt\;\]/<div align="center">/ig;
		${$text}=~ s/\[\&lt\;\&gt\;\]/<div align="justify">/ig;
		${$text}=~ s/\[\/\&[lgt\;].+?\;\]/<\/div>/ig;
		${$text}=~ s/\<\/div\>\n/\<\/div\>/g;
		${$text}=~ s/\[\*\]/&nbsp;&nbsp;<b>&middot;<\/b>&nbsp;/g;
		${$text}=~ s/\[\+\]/<span style="font-size:120%;">/g;
		${$text}=~ s/\[\-\]/<span style="font-size:80%;">/g;
		${$text}=~ s/\[\/[\+\-]\]/<\/span>/g;
		${$text}=~ s/\x20\x20/\x20&nbsp;/g;
		${$text}=~ s/\n/<br \/>/g;
		${$text}=~ s/\r//g;
		return 0;
	} else {
		return undef;
	}
}

sub linkme
{
	my $fulllink = shift;
	my $cutlink  = shift;
	my $endlink  = shift;
	$fulllink =~ s/([^\w\=\-\/]*)$//;
	my $erase = $1;

	my $test = qq|<a href="$fulllink" class="$linkstyle">|;
	if ($endlink eq "") {
		$test.=$fulllink."</a>$erase";
	} else {
		$test.=$cutlink."&hellip;</a>$erase";
	}

	return $test;
}

sub MakeDate{
	my ($class,$time) = self_or_default(@_);
	my @t = ($time =~ /^(\d+)-(\d+)-(\d+)/);
	$t[2] += 0;
	$time = $t[2]." ".$month[$t[1]-1]." ".$t[0];
	return $time;
}

sub MakeTime{
	my ($class,$time) = self_or_default(@_);
	my @t = ($time =~ /\s(\d+)\:(\d+)\:\d+$/);
	$time = $t[0].":".$t[1];
	return $time;
}

sub MakeDateTime{
	my ($class,$time) = self_or_default(@_);
	if ($time eq "") { return undef; }
	my @t = ($time=~/(\d+)-(\d+)-(\d+)\s(\d+):(\d+):(\d+)/);
	$t[2] += 0;
	$time = $t[2]." ".$month[$t[1]-1]." ".$t[0].", $t[3]:$t[4]";
	return $time;
}

sub isInArray{
    my ($class,$element,$array) = self_or_default(@_);
    my $match = 0;

    if (ref($array) && defined($element)){
        foreach (@{$array}){
	    if ($element eq $_){
		$match = 1;
		last;
	    }
	}
	return $match;
    } else {
	return undef;
    }
}

sub messages {
    my ($class,$message) = self_or_default(@_);
    if (exists $messages{$message}) {
	    return $messages{$message};
    } else {
    	return "undefined error";
    }
}

sub transPath {
	my $path = shift;
	my @par = ();
	my @return = ();
	
	if (! defined($$path) || length($$path) < 1 || length($$path) > 150){
		return @return;
	}

	foreach ( split(/\//, $$path) ) {
		if (defined($_) && $_ =~ /^[\w\.\-\+]+$/ ) {
			push @par,$_;
		}
	}

	my $i = 0; 
	foreach my $control (@_) {
		unless(defined($par[$i])) {
			last;
		}
		if (ref($control) eq "ARRAY"){
			if (isInArray($par[$i],$control)){
				push @return,$par[$i];
			} else {
				push @return, undef;
			}
		} elsif (! ref($control)) {
			# "w" - all good symbols
			if ($control eq "w"){
				push @return,$par[$i];
			# "s" - "w" without "+" & "."
			} elsif ($control eq "s"){
				if ($par[$i] =~ /^[\w\-]+$/){
					push @return,$par[$i];
				} else {
					push @return, undef;
				}
			# "d" - only digits
			} elsif ($control eq "d"){
				if ($par[$i] =~ /\D/) {
					push @return, undef;
				} else {
					push @return,$par[$i];
				}
			}
		} else {
			push @return, undef;
		}
		$i++;
	}
	return @return;
}

sub strBreak {
	my $string = shift;
	my $length = shift;
	if (length($string) < $length || $length < 1) {
		return $string;
	}
	for (my $i=1; $i <= (length($string)/$length); $i++){
		substr $string,$i*$length,0," ";
	}
	return $string;
}

sub localeGrp {
	my $group = shift;
	if (exists $totalGroups{$group}){
		$group = $totalGroups{$group};
	}
	return $group;
}

sub pathAlias {
	my $repo = shift;
	SWITCH: {
    	  if (($repo eq "sisyphus") || ($repo eq "s")) { $repo="Sisyphus"; last SWITCH; }
    	  if (($repo eq "daedalus") || ($repo eq "d")) { $repo="Daedalus"; last SWITCH; }
	  if (($repo eq "branch5") || ($repo eq "5")) { $repo="Branch5"; last SWITCH; }
	  if (($repo eq "branch41") || ($repo eq "4.1"))  { $repo="Branch41" ; last SWITCH; }
    	  if (($repo eq "branch4") || ($repo eq "4"))  { $repo="Branch4" ; last SWITCH; }
    	  if ($repo eq "up4") { $repo="UP4"; last SWITCH; }
    	  if ($repo eq "bp4") { $repo="BP4"; last SWITCH; }
    	  if (($repo eq "branch3") || ($repo eq "3"))  { $repo="Branch3" ; last SWITCH; }
    	  if ($repo eq "up3") { $repo="UP3"; last SWITCH; }
    	  if ($repo eq "bp3") { $repo="BP3"; last SWITCH; }
	}
	return $repo;
}

sub newStyle {
	my $newstyle;
	my @par=@_;
	$par[0]=altbase::pathAlias($par[0]);
	foreach my $repo (@altbase::repos) {
		if ($repo eq $par[0]) { 
		    $newstyle = 1;
		    last; 
    		}
	}
	if (! defined($newstyle)) {
	    for (my $n=scalar(@par);$n>0;$n--){
	    	$par[$n]=$par[$n-1];
	    }
	    $par[0] = "Sisyphus"; # default repo
    	}
	return @par;
}

1;
