#!/usr/bin/perl
#
# script:       find.pl
# description:  search in database
# license:	GPL v2 or later
# authors:      Vladimir Lettiev aka crux <crux@altlinux.ru> (c) 2004
#		Andrew Avramenko <liks@altlinux.ru> (c) 2007 
# version:      0.2

#
# TODO: - another way of sorting rpms - by relevance: first if equal to name, then summary, then description.
#         

use strict;
use CGI qw/param/;
use DBI;
use FindBin qw($Bin);
use lib "$Bin/adm";
use altbase;

#> My constants
my ($select_Q,@row,$row,$sth,$dbh);
my $text = "";
my $subtext = "";
my $cgi = new CGI;
my $alt = new altbase;

#> Load html fragments
my $lang = $cgi->param('lang');
if (! $alt->init(module => "inform", lang => $lang, script => "find.pl") ){
    print $altbase::header . $altbase::errStr;
    exit;
}

my $request = $cgi->param('request');
my $page = $cgi->param('page');

if (! defined($page) || $page =~ /\D/ || $page > $altbase::maxPage || $page < 1) {
    $page = 1;
}

my $messPerPage = 20;
my $startPage = ($page-1)*$messPerPage;


if (defined($request) && length($request) > 1) {

    if (length($request) > 50){
	$request = substr($request,0,50);
    }

    my @arr = split(/\s+/,$request);
    my @find = ();

    for (my $i=0; $i <= $#arr; $i++){
	if (length($arr[$i]) > 1 && $i < 6) {
	    push @find, $arr[$i];
	}
    }

    undef @arr;

    if (@find) {
	#> Connection to MySQL
        $dbh = DBI->connect($altbase::dsn,$altbase::dbUser,$altbase::dbPass);

        # Search SRPM

	my @temp = ();
	my @temp2;
	my @q = ();
	my $j = 0;
	
    
    	foreach my $what ("s.name", "s.summary", "s.description", "r.name", "r.description") {
	    @temp2 = ();
	    for (my $i=0; $i <= $#find; $i++){
		push @temp2, " $what RLIKE ? ";
		push @q, quotemeta($find[$i]);
	    }
	    push @temp, join(" AND ",@temp2);
	}

	# Find packages in Sisyphus repo
        my $findRPM_Q = "SELECT DISTINCT s.name, s.version, s.rel, m.packager, s.summary, s.repo FROM srpm as s, maintainers as m, rpm as r " .
			"WHERE  r.srpm = s.name AND  m.mail=s.packager AND (". join(" OR ",@temp) .") AND s.repo='Sisyphus' AND r.repo='Sisyphus' " .
			"ORDER BY 1 ASC LIMIT $startPage,$messPerPage";
        $sth = $dbh->prepare($findRPM_Q);
        $sth->execute(@q);
	rpmoutput($sth);

	# Find packages in orphaned and obsolete. Difference - no binary rpms.
	my $findRPM_Q = "SELECT DISTINCT s.name, s.version, s.rel, m.packager, s.summary, s.repo FROM srpm as s, maintainers as m WHERE (s.repo='orphaned' ".
			"OR s.repo='obsolete') AND m.mail=s.packager AND s.name RLIKE '@find' ORDER BY 1 ASC LIMIT $startPage,$messPerPage";
        $sth = $dbh->prepare($findRPM_Q);
        $sth->execute();
	rpmoutput($sth);

	# Count Sisyphus SRPMS
        my $findRPMcount_Q = "SELECT count(DISTINCT s.name) FROM srpm as s, rpm as r WHERE r.srpm = s.name " . 
			      "AND s.repo='Sisyphus' AND r.repo='Sisyphus' AND (". join(" OR ",@temp) .")";
        $sth = $dbh->prepare($findRPMcount_Q);
        $sth->execute(@q);
	$row = $sth->fetchrow_array();
	
	# Count orphaned and obsolete SPRMS
	my $findRPMcount_Q = "SELECT count(DISTINCT s.name) FROM srpm as s WHERE s.name RLIKE '@find' AND (s.repo='orphaned' OR s.repo='obsolete')";
        $sth = $dbh->prepare($findRPMcount_Q);
	$sth->execute(@q);
	$row += $sth->fetchrow_array();
	$sth->finish();
	
        if ($row > $messPerPage) {
            my $pg = sprintf("%u", ($row/$messPerPage));
	    if ($row%$messPerPage != 0 ) {
		$pg++;
	    }
	    for (my $i=1; $i<=$pg; $i++){
		if  ($pg > 20 && $i==1) {
		    $subtext .= "<br />";
		}
		$subtext .= $alt->parser(
	                   HTML => "link2page",
	                   vars => {
	                       "path"=>$altbase::path,
	                       "page"=>($pg > 20)?(($i>9)?$i:"0$i"):$i,
	                       "request"=>urlEscape($request),
	                       "lang" => $lang,
	                   },
	                   iflist => {
	                       "isCur" => ($i==$page)?1:0
	                   }
		);
		if ($i%20 == 0) {
		    $subtext .= "<br />";
		}
	    }
	}
        $dbh->disconnect();
    }
}



$text = $alt->parser(
            HTML => "main",
            vars => {
		"content" => $text,
		"count" => $row,
		"request" => tagClean($request),
		"pages" => $subtext
            },
            iflist => {
		"find" => $text?1:0,
		"pages" => $subtext?1:0
	    }
);


print $altbase::header;
print $text;

sub rpmoutput {
	$sth = shift;
	
        while (@row = $sth->fetchrow_array){

	    $row[3] =~ s/^(.+?)\s<.+$/$1/;

            $text .= $alt->parser(
	            HTML => "rpm",
    	            vars => {
			"lang" => $lang,
			"name" => $row[0],
			"version" => $row[1],
			"release" => $row[2],
			"packager" => tagClean($row[3]),
			"summary" => tagClean($row[4]),
			"repo" => $row[5],
			"path" => $altbase::path
                    },
	            iflist => {}
		);
	}

}
