#!/usr/bin/perl
#
# script:       groups.pl
# description:  show information about RPM groups
# author:       Vladimir Lettiev aka crux <crux@altlinux.ru> (c) 2004
# version:      0.1

use strict;
use CGI qw/param/;
use DBI;
use FindBin qw($Bin);
use lib "$Bin/adm";
use altbase;

#> My constants
my ($select_Q,@row,$row,$sth,$dbh);

my $text = "";

my $cgi = new CGI;
my $alt = new altbase;

#> Load html fragments
my $lang = $cgi->param('lang');
if (! $alt->init(module => "inform", lang => $lang, script => "groups.pl") ){
    print $altbase::header . $altbase::errStr;
    exit;
}

my (@grpId) = &transPath(\$ENV{"PATH_INFO"},"w","w","w");
my $grpId = undef;
if (@grpId) {
	$grpId = join("/",@grpId);
	$grpId =~ s/_/ /g;
}
my $show = $cgi->param('show');

#> Connection to MySQL
$dbh = DBI->connect($altbase::dsn,$altbase::dbUser,$altbase::dbPass);

unless(defined($grpId)){

	$text = $alt->parser(
		HTML => "main",
		vars => {
			"grplist" => &showTree()
		},
		iflist => {}
	);

	print $altbase::header;
	print $text;
	exit;
}

my $selGrpName_Q = "SELECT id FROM groups WHERE rpmgroup=?";

my $selRpm_Q = "SELECT s.name, s.version, s.rel, m.packager, s.buildtime, s.summary, ".
    "c.content, s.packager, m.type, UNIX_TIMESTAMP(NOW())-UNIX_TIMESTAMP(s.buildtime) " .
    "FROM srpm as s, maintainers as m, changelogs as c WHERE m.mail=s.packager " .
    "AND s.rpmgroup=? AND c.rpm=s.name AND s.repo='Sisyphus' AND c.repo='Sisyphus' ".
    "AND c.version=CONCAT(s.version,'-',s.rel) ORDER by s.name ASC";

$sth = $dbh->prepare($selGrpName_Q);
$sth->execute($grpId);
my $group = $sth->fetchrow_array();
unless ($group){
	print $altbase::header;
	print "error: groups.pl:61";
	exit;
}

$text = $alt->parser(
	HTML => "curgroup",
	vars => { "group" => &localeGrp($grpId) },
	iflist => {}
);

$sth = $dbh->prepare($selRpm_Q);
$sth->execute($group);
while (@row = $sth->fetchrow_array){
	$row[3] =~ s/\s*<.+$//;
	tagConv(\$row[6]);
	$text .= $alt->parser(
		HTML => "package",
		vars => {
			"lang" => $lang,
			"path" => $altbase::path,
			"name" => $row[0],
			"version" => $row[1],
			"release" => $row[2],
			"mail" => $row[7],
			"packager" => tagClean($row[3]),
			"date" => MakeDateTime($row[4]),
			"oldness" => sprintf("%.1f",$row[9]/(3600*24*7)),
			"summary" => tagClean($row[5]),
			"changes" => $row[6]
		},
		iflist => {
			"team" => ($row[8] eq "team")?0:1
		}
	);
}

print $altbase::header;
print $text;

sub showTree {

	my (@oldgrp,@groups,$id,%TH,@row,$sth,$text);

	$sth = $dbh->prepare("SELECT count(srpm.name), groups.id, groups.rpmgroup FROM groups,srpm WHERE groups.id=srpm.rpmgroup AND srpm.repo='Sisyphus' GROUP BY groups.id ORDER BY groups.rpmgroup ASC");
	$sth->execute();

	while (@row = $sth->fetchrow_array){
		$TH{&localeGrp($row[2])} = [@row];
	}
	foreach my $r ( sort keys %TH) {
		@row = @{$TH{$r}};
		@groups = reverse split("\/",&localeGrp($row[2]));

		if ($#oldgrp > ($#groups-1)) {
			$text .= "</div>"x($#oldgrp - $#groups + 1);
		} elsif (($#oldgrp == ($#groups-1)) && ($oldgrp[0] ne $groups[1])){
			$text .= "</div>";
		}
		
		$row[2] =~ s/\s/_/g;
		
		if ($#groups == 0 || isInArray($groups[1],\@oldgrp)){
			$text .= $alt->parser(
				HTML => "group",
				vars => {
					"count" => $row[0],
					"grp_id" => $row[2],
					"group" => $groups[0]
				},
				iflist => {}
			);
		} else {
			$text .= $alt->parser(
				HTML => "maingroup",
				vars => {
					"count" => $row[0],
					"grp_id" => $row[2],
					"group" => $groups[0],
					"dirgroup" => $groups[1],
					"id" => $id++
				},
				iflist => {}
			);
		}

		@oldgrp = @groups;
		shift(@oldgrp);
	}

	if ($#oldgrp >= 0) {
		$text .= "</div>"x($#oldgrp + 1);
	}
	return $text;
}
