#!/usr/bin/perl
#
# script:       packager.pl
# description:  show information about maintainer
# author:       Vladimir Lettiev aka crux <crux@altlinux.ru> (c) 2004-2005
# version:      0.02

use strict;
use CGI qw/param/;
use DBI;
use FindBin qw($Bin);
use lib "$Bin/adm";
use altbase qw(:DEFAULT $repocopSupport);
#$repocopSupport=0;

#> My constants
my ($select_Q,@ary,@row,$row,$sth,$dbh);
my $text = "";
my @sections = ("main","srpms","bugs","allbugs","repocop");

my $cgi = new CGI;
my $alt = new altbase;

#> Load html fragments
my $lang = $cgi->param('lang');
if (! $alt->init(module => "inform", lang => $lang, script => "packager.pl") ){
	print $altbase::header . $altbase::errStr;
	exit;
}

my ($packager,$section) = transPath(\$ENV{"PATH_INFO"},"s",\@sections);
my $show = $cgi->param('show');

my $selMainInfo_Q = "SELECT packager, type FROM maintainers WHERE mail=?";

my %repocopStatusTxt = (
    0 => 'internal error',
    1 => 'skip',
    2 => 'ok',
    3 => 'experimental',
    4 => 'info',
    5 => 'warn',
    6 => 'fail',
    );

unless ( defined($packager) ){
	print $altbase::header;
	print (($show eq "title")?"unknown":"error [packager.pl:38]");
	exit;
}

my $sort = $cgi->param('sort');
my $order = $cgi->param('order');
if ( ! defined($sort) || (($sort ne "age") && ($sort ne "name") && ($sort ne 'status')) ){
    print STDERR "$$:strange sort=$sort; killed\n";
    $sort = "name";
    $order = "asc";
} elsif (! defined ($order) || ($order ne "asc" && $order ne "desc")){
    $order = "asc";
}

#> Connection to MySQL
$dbh = DBI->connect($altbase::dsn,$altbase::dbUser,$altbase::dbPass);

if ($packager eq "nobody" && $show ne "title") {
    my ($srpmtable,$count) = &srpmListTable($packager,$sort,$order,"nobody");
    
    $text = $alt->parser(
		HTML => "team",
		vars => {
			"members" => "",
			"packager" => "Nobody",
			"info" => "",
			"srpmtable" => $srpmtable,
			"count" => $count,
		},
		iflist => {}
	);
}
else {
    $sth = $dbh->prepare($selMainInfo_Q);
    $sth->execute($packager);
    @row = $sth->fetchrow_array();
    $sth->finish();
    unless (@row){
	print $altbase::header;
	print (($show eq "title")?"$packager":"error [packager.pl:48]");
	exit;
    }

    if ($show eq "title"){
	print $altbase::header;
	print $packager;
	exit;
    }

    $row[0] =~ s/\s*<.+$//;

    my ($srpmtable,$count) = &srpmListTable($packager,$sort,$order);

    if ($row[1] eq "team") {
	$text = $alt->parser(
		HTML => "team",
		vars => {
			"members" => &showList($packager),
			"packager" => $row[0],
			"info" => "",
			"srpmtable" => $srpmtable,
			"count" => $count,
		},
		iflist => {}
	);
    } else {
	unless ( defined($section) ) {
		$sth = $dbh->prepare("SELECT sirname, name, o, info, work, born, city, country " .
			"FROM members WHERE mail=?" );
		$sth->execute($packager);
		my @info = $sth->fetchrow_array();
		$sth->finish();

		my $info = "";

		if ($info[0]) {
			$info = $alt->parser(
				HTML=>"anketa",
				vars => {
					name => tagClean($info[1]),
					o => tagClean($info[2]),
					sirname => tagClean($info[0]),
					country => tagClean($info[7]),
					city => tagClean($info[6]),
					work => tagClean($info[4]),
					info => tagClean($info[3]),
					born => MakeDate($info[5])
				},
				iflist => {}
			);
		}
		
		$text = $alt->parser(
			HTML => "packager",
			vars => {
				"packager" => $row[0],
				"info" => $info,
				"count" => $count
			},
			iflist => {}
		);

	} elsif ($section eq "srpms"){

		$text = $alt->parser(
			HTML => "rpmlist",
			vars => {
				"srpmtable" => $srpmtable,
				"count" => $count,
			},
			iflist => {}
		);

        } elsif ($section eq "bugs" || $section eq "allbugs"){

		my $statusFilter = "";
		my @statusFilter = ();
		my %countBugs = ();
		
		foreach ('UNCONFIRMED','NEW','ASSIGNED','REOPENED') {
		    push @statusFilter,"bugs.status='$_'";
		}
		$statusFilter = " AND (" . join(' OR ',@statusFilter) . ")";

		my $selBugsList_Q = "SELECT srpm.name, count(bugs.bug_id), srpm.summary " .
		"FROM bugs, rpm, srpm, maintainers WHERE bugs.assigned=? AND rpm.repo='Sisyphus' " .
		"AND srpm.repo='Sisyphus' AND rpm.name=bugs.rpm AND rpm.srpm=srpm.name AND maintainers.mail=srpm.packager ".
		"AND ((srpm.packager!=bugs.assigned AND maintainers.type='team') OR (srpm.packager=bugs.assigned)) ";
		$selBugsList_Q .= $statusFilter if ($section eq "bugs");
		$selBugsList_Q .= " GROUP BY srpm.name ORDER BY 2 DESC, srpm.name ASC";

		$sth = $dbh->prepare($selBugsList_Q);
		$sth->execute($packager);
		
		my $i = 0;
		my $j = 0;
		while( @ary = $sth->fetchrow_array ){
			$i+=$ary[1];
			$j++;
			$text .= $alt->parser(
				HTML => "bug",
				vars => {
					"lang" => $lang,
					"path" => $altbase::path,
					"rpm" => $ary[0],
					"count" => $ary[1],
					"summary" => tagClean($ary[2])
				},
				iflist => {}
			);
		}
		
		my $selBugsCount_Q = "SELECT count(bugs.bug_id) " .
		"FROM bugs, rpm, srpm, maintainers WHERE bugs.assigned=? AND rpm.name=bugs.rpm " .
		"AND rpm.repo='Sisyphus' AND srpm.repo='Sisyphus' AND rpm.srpm=srpm.name AND maintainers.mail=srpm.packager " .
		"AND ((srpm.packager!=bugs.assigned AND maintainers.type='team') OR (srpm.packager=bugs.assigned)) ";

		$selBugsCount_Q .= $statusFilter if ($section eq "allbugs");
		
		$sth = $dbh->prepare($selBugsCount_Q);
		$sth->execute($packager);
		$countBugs{"get"} = $sth->fetchrow_array();
		$sth->finish();
		
		if ($section eq "bugs") {
		    $countBugs{"allBugsCount"}  = $countBugs{"get"};
		    $countBugs{"openBugsCount"} = $i;
		} else {
			$countBugs{"allBugsCount"}  = $i;
			$countBugs{"openBugsCount"} = $countBugs{"get"};
		}
																							    
		
		$text = $alt->parser(
			HTML => "bugslist",
			vars => {
				"lang" => $lang,
				"content" => $text,
				"count" => $i,
				"rpmcnt" => $j,
				"name" => $packager,
				"allBugsCount" => $countBugs{"allBugsCount"},
				"openBugsCount" => $countBugs{"openBugsCount"},
				"closedBugsCount" => $countBugs{"allBugsCount"} - $countBugs{"openBugsCount"}
			},
			iflist => {
				"bugs" => ($countBugs{"allBugsCount"} == 0)?0:1,
				"open" => ($section eq "bugs" && $countBugs{"allBugsCount"} >0 )?1:0,
				"all" => ($section eq "allbugs" && $countBugs{"allBugsCount"} >0 )?1:0
			}
		);
	} elsif ($section eq "repocop"){
		my $selRepocop_Q = "SELECT CONCAT(repocop.name,'-',repocop.version,'-',repocop.rel,'.',repocop.arch) as repocopid, repocop.srcname, repocop.status, repocop.testname, repocop.message FROM repocop LEFT JOIN srpm ON repocop.srcname=srpm.name WHERE repocop.status<>'skip' AND repocop.status<>'ok' AND srpm.packager=? AND srpm.repo='Sisyphus'";
		$selRepocop_Q .= " ORDER BY repocop.name ASC, repocop.status DESC, repocop.version DESC, repocop.rel DESC, repocop.arch DESC";
			
		my $countRepocop=0;
		$sth = $dbh->prepare($selRepocop_Q) || die $dbh->errstr;
		$sth->execute($packager) || die $dbh->errstr;
		while (@ary = $sth->fetchrow_array){
			$countRepocop++;
			$text .= $alt->parser(
				HTML => "repocopmsg",
				vars => {
					"lang" => $lang,
					"id" => $ary[0],
					"name" => $ary[1],
                               		"status" => $ary[2],
					"testname" => $ary[3],
					"message" => $ary[4],
				},
				iflist => {}
			);
		}
			

		$text = $alt->parser(
			HTML => "repocoplist",
			vars => {
				"content" => $text,
			},
			iflist => {
				"repocopmsgs" => $countRepocop,
				"repocopnomsgs" => (not $countRepocop),
				"repocopsupport" => $repocopSupport,
			}
		);

	} else {
		# NOTHING HERE
	}

	$text = $alt->parser(
		HTML => "titleman",
		vars => {
			"lang" => $lang,
			"packager" => $row[0],
			"mail" => $packager,
			"path"=> $altbase::path,
			"content"=> $text
		},
		iflist => {
			"main" => (! defined($section))?1:0,
			"packages" => ($section eq "srpms")?1:0,
			"bugs" => ($section eq "bugs" || $section eq "allbugs")?1:0,
			"repocop" => ($section eq "repocop")?1:0,
		}
	);
    }
}
$dbh->disconnect();
print $altbase::header;
print $text;

sub rpmList {
	my $mail = shift;
	my $sort = shift;
	my $order = shift;
	my $type = shift; # type can be only 'nobody' or undef now
	
	my ($sth,@row,$text,$count,$url,$selMainRPM_Q);
	my $sqlsort = 1;
	print STDERR "$$:NOTE: sort=$sort order=$order\n";
	$sqlsort = 3 if $sort eq "age";
	$sqlsort = 7 if $sort eq "status" and $repocopSupport;
	my $selectEXPR="s.name, CONCAT(s.version,'-',s.rel), UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP(s.buildtime), " .
	    "s.buildtime, s.url, s.summary";
	my $tableEXPR='srpm as s';
	my $groupByEXPR='';
	if ($repocopSupport) {
	    $tableEXPR='srpm as s LEFT JOIN repocop as r ON r.srcname=s.name AND r.srcversion=s.version AND r.srcrel=s.rel LEFT JOIN repocop_patches as p ON p.name=s.name AND p.version=s.version AND p.rel=s.rel';
	    $selectEXPR .= ', MAX(r.status+0), p.url';
	    $groupByEXPR='GROUP BY s.name';
	}
	if ($type eq "nobody") {
	    $selMainRPM_Q = "SELECT $selectEXPR FROM $tableEXPR, acl as a WHERE s.repo='Sisyphus' AND s.name=a.srpm AND a.packager='\@nobody' $groupByEXPR " . 
		"ORDER BY $sqlsort $order";
	}
	else {
	    $selMainRPM_Q = "SELECT $selectEXPR FROM $tableEXPR WHERE s.packager=? AND s.repo='Sisyphus' $groupByEXPR ORDER BY $sqlsort $order";
	}
	print STDERR "$selMainRPM_Q\n";
	$sth = $dbh->prepare($selMainRPM_Q) || die "INTERNAL: ".$dbh->errstr;
	if ($type eq "nobody") {
	    $sth->execute() || die "INTERNAL: ".$dbh->errstr;
	}
	else {
	    $sth->execute($mail) || die "INTERNAL: ".$dbh->errstr;
	}
	while(@row = $sth->fetchrow_array()){
		$count++;
		tagConv(\$row[4],20);
		$text .= $alt->parser(
			HTML => "srpm",
			vars => {
				"lang" => $lang,
				"path" => $altbase::path,
				"name" => $row[0],
				"subName" => strBreak($row[0],20),
				"release" => strBreak($row[1],20),
				"date" => MakeDateTime($row[3]),
				"old" => sprintf("%.1f",$row[2]/(3600*24*7)),
				"url" => ($row[4])?$row[4]:"-",
				"summary" => tagClean($row[5]),
				"repocop" => $row[6]?$repocopStatusTxt{$row[6]}:'skip',
				"patchurl" => $row[7],
    		    },
			iflist => {
				"repocopsupport" => $repocopSupport,
				"patchurl" => $row[7]? 1:0,
			}
		);
	}
	return $text,$count;
}

sub srpmListTable {
    my ($packager,$sort,$order,$type)=@_;
    my ($rpmlist,$count) = &rpmList($packager,$sort,$order,$type);
    my $srpmtable = $alt->parser(
	HTML => "srpmtable",
	vars => {
	    "rpmlist" => $rpmlist,
	    "count" => $count,
	    "statusOrder" => ($sort eq "status" && $order eq "desc")?"asc":"desc",
	    "nameOrder" => ($sort eq "name" && $order eq "asc")?"desc":"asc",
	    "ageOrder" => ($sort eq "age" && $order eq "asc")?"desc":"asc",
	    "statusDir" => ($sort eq "status" && $order eq "desc")?"next":"prev",
	    "nameDir" => ($sort eq "name" && $order eq "asc")?"prev":"next",
	    "ageDir" => ($sort eq "age" && $order eq "asc")?"prev":"next"
	},
	iflist => {
	    "repocopsupport" => $repocopSupport,
	}
	);
    return ($srpmtable, $count);
}

sub showList {
	my $mail = shift;
	my ($text,$sth,$row,$packager,@row);

	$sth = $dbh->prepare("SELECT DISTINCT m.packager, m.type, m.mail " .
	"FROM maintainers as m, changelogs as c, srpm as s " .
	"WHERE s.packager=? AND s.name=c.rpm AND m.mail=c.packager");
	$sth->execute($mail);

	while (@row = $sth->fetchrow_array()){
		$packager = ( $row[0] =~ /^(.+?)\</ )[0];
		$text .= $alt->parser(
			HTML => "member",
			vars => {
				"lang" => $lang,
				"path" => $altbase::path,
				"mail" => $row[2],
				"packager" => $packager,
			},
			iflist => {
				"team" => ($row[1] eq "team")?0:1
			}
		);
	}
	$sth->finish();

	return $text;
}

