#!/usr/bin/perl

use strict;
use CGI qw/param/;
use DBI;
use FindBin qw($Bin);
use lib "$Bin/adm";
use altbase;

my $cgi = new CGI;
my $alt = new altbase;
my @sections = ("bad_elf_symbols","unmets","missassigned_bugs", "rebuild_status");

my ($dbh,$sth,@row);
my $text = "";
my $linkText = "";
my $begin = 0;

#> Load html fragments
my $lang = $cgi->param('lang');
if (! $alt->init(module => "inform", lang => $lang, script => "qa.pl") ){
	print $altbase::header . $altbase::errStr;
	exit;
}

#> Connection to MySQL
$dbh = DBI->connect($altbase::dsn,$altbase::dbUser,$altbase::dbPass);

my ($section,$num) = transPath(\$ENV{"PATH_INFO"},\@sections,"d");
my $show = $cgi->param("show");

if ($show eq "list") {

	foreach my $sect (@sections) {
		$linkText .= $alt->parser(
			HTML => "sectLink",
			vars => {
				"lang" => $lang,
				"path" => $altbase::path,
				"link" => $sect
			},
			iflist => {
				"current" => ($sect eq $section)?1:0
			}
		);
	}
	print $altbase::header;
	print $linkText;
	exit;
}
			
unless (defined($section) && isInArray($section,\@sections)){

	$text = $alt->parser(HTML=>"intro", vars=>{}, iflist => {});

} else {
	
	$sth = $dbh->prepare("SELECT count(*) FROM qarobot WHERE report_name=?");
	$sth->execute($section);
	my $count = $sth->fetchrow_array();
	
	if ($count) {
		($begin, $linkText) = &links($count,$section,\$num);
	
		$sth = $dbh->prepare("SELECT time, rawdata from qarobot WHERE report_name=? ORDER BY time DESC LIMIT $begin,1");
		$sth->execute($section);
		@row = $sth->fetchrow_array();
		my ($numPlus, $numMinus, $contentPlus, $contentMinus);

		foreach my $line ( split(/\n/,$row[1]) ){
			my @data = split(/\t/,$line);
			my $act = shift @data;
			if ($act eq "+"){
				$contentPlus .= "<tr><td>". join ("</td><td>", @data)."</td></tr>";
				$numPlus++;
			} elsif ($act eq "-") {
				$contentMinus .= "<tr><td>". join ("</td><td>", @data)."</td></tr>";
				$numMinus++;
			}
		}
	
		$text = $alt->parser(
			HTML => $section,
			vars => {
				"lang" => $lang,
				"time" => MakeDateTime($row[0]),
				"numPlus" => $numPlus,
				"numMinus" => $numMinus,
				"contentPlus" => $contentPlus,
				"contentMinus" => $contentMinus
			},
			iflist => {
				"isMinus" => ($numMinus > 0)?1:0,
				"isPlus" => ($numPlus >0)?1:0
			}
		);
	}
}

print $altbase::header;
print $alt->parser(
	HTML => "main",
	vars => {
		"content" => $text,
		"pages" => $linkText
	},
	iflist => {
		"pages" => ($linkText)?1:0
	}
);

$dbh->disconnect();

sub links {
	my ($count,$section,$num) = @_;
	my $linkText = "";
	my $begin = 0;
	
	if ($$num == 0 || $$num > $count) {
		$$num = $count;
	} else {
		$begin = $count - $$num;
	}
	
	for (my $i=$count; $i>= 1; $i--) {
		$linkText .= $alt->parser(
			HTML => "link2report",
			vars => {
				"lang" => $lang,
				"page" => $i,
				"path" => $altbase::path,
				"section" => $section
			},
			iflist => {
				"current" => ($i==$$num)?1:0
			}
		);
	}

	return ($begin,$linkText);
}
