#!/usr/bin/perl
#
# script:      repostats.pl
# description: show information about version of SRPM in other repos
# author:      Andrew Avramenko aka liks <liks@altlinux.ru> (c) 2007
# 
# comments:    type is type of repo. Now implemeted only ALT. I hope
#              in future we can look which version in upstream or in 
#              other repos. This option is in TODO stage now.

use strict;
use CGI qw/param/;
use DBI;
use FindBin qw($Bin);
use lib "$Bin/adm";
use altbase;

#> My constants
my ($select_Q,$name,$dbh,$sth,@row,%version);
my $text = "";

my $cgi = new CGI;
my $alt = new altbase;
my $type = param('type');

my ($repo,$srpm,$section,$extra) = altbase::newStyle(transPath(\$ENV{"PATH_INFO"},"w","w","w","d"));

print $altbase::header;
#> Load html fragments
my $lang = $cgi->param('lang');
if (! $alt->init(module => "inform", lang => $lang, script => "repostats.pl") ){
	print $altbase::errStr;
	exit;
}

#> Check if all parameters given
if (!defined($type) || !defined($srpm)) {
    print "Sorry. No enough parameters given.";
    exit;
}

$dbh = DBI->connect($altbase::dsn,$altbase::dbUser,$altbase::dbPass);

if ($type eq "alt") {
	$select_Q = "SELECT repo,version,rel FROM srpm WHERE name='$srpm'";
	$sth = $dbh->prepare($select_Q);
	$sth->execute();

	while (@row = $sth->fetchrow_array){
	    $version{$row[0]} = $row[1] . "-" . $row[2];
	}
	
	$text .= $alt->parser(
		HTML => "repoList",
		vars => {
			"lang" => $lang,
			"nameoftype" => "ALT",
			"name" => $srpm,
			"path" => $altbase::path,
			"sisyphusver" => $version{'Sisyphus'},
			"daedalusver" => $version{'Daedalus'},
			"branch5ver" => $version{'Branch5'},
			"branch41ver" => $version{'Branch41'},
			"up41ver" => $version{'UP41'},
			"bp41ver" => $version{'BP41'},
			"branch4ver" => $version{'Branch4'},
			"up4ver" => $version{'UP4'},
			"bp4ver" => $version{'BP4'},
			"branch3ver" => $version{'Branch3'},
			"up3ver" => $version{'UP3'},
			"bp3ver" => $version{'BP3'},
			"orphver" => $version{'orphaned'},
			"obsver" => $version{'obsver'},
			"target" => $section . $extra
		},
		iflist => {
			"sisyphus" => (defined($version{'Sisyphus'}))?1:0,
			"daedalus" => (defined($version{'Daedalus'}))?1:0,
			"branch5" => (defined($version{'Branch5'}))?1:0,
			"branch41" => (defined($version{'Branch41'}))?1:0,
			"updates41" => (defined($version{'UP41'}))?1:0,
			"backports41" => (defined($version{'BP41'}))?1:0,
			"branch41empty" => ((defined($version{'BP41'}) || defined($version{'UP41'})) && !defined($version{'Branch41'}))?1:0,
			"branch4" => (defined($version{'Branch4'}))?1:0,
			"updates4" => (defined($version{'UP4'}))?1:0,
			"backports4" => (defined($version{'BP4'}))?1:0,
			"branch4empty" => ((defined($version{'BP4'}) || defined($version{'UP4'})) && !defined($version{'Branch4'}))?1:0,
			"branch3" => (defined($version{'Branch3'}))?1:0,
			"updates3" => (defined($version{'UP3'}))?1:0,
			"backports3" => (defined($version{'BP3'}))?1:0,
			"branch3empty" => ((defined($version{'BP3'}) || defined($version{'UP3'})) && !defined($version{'Branch3'}))?1:0,
			"orphaned" => (defined($version{'orphaned'}))?1:0,
			"obsolete" => (defined($version{'obsolete'}))?1:0
		}
	);

} 
elsif ($type eq "upstream") {
    $select_Q = "SELECT version FROM other WHERE srpm='$srpm' AND repo='freshmeat'";
    $sth = $dbh->prepare($select_Q);
    $sth->execute();

    @row = $sth->fetchrow_array();
    if (@row != 0) {
	$text .= $alt->parser(
	    HTML => "oneOtherRepo",
	    vars => { 
		"version" => @row,
		"url" => "http://www.freshmeat.net/projects/$srpm",
		"repo" => "Freshmeat"
	    },
	    iflist => {}
	);
    }
    
    $select_Q = "SELECT version,url FROM other WHERE srpm='$srpm' AND repo='upstream'";
    $sth = $dbh->prepare($select_Q);
    $sth->execute();

    @row = $sth->fetchrow_array();
    if (@row != 0) {
	$text .= $alt->parser(
	    HTML => "oneOtherRepo",
	    vars => { 
		"version" => $row[0],
		"url" => $row[1],
		"repo" => "Upstream"
	    },
	    iflist => {}
	);
    }
    
    if ($text ne "") {
	$text = $alt->parser(
	    HTML => "otherRepoList",
	    vars => { "content" => "$text"},
	    iflist => {}
	);
    }
    else {
	print "</div>";
    }
}

$sth->finish();
$dbh->disconnect();

print $text;
