#!/usr/bin/perl
#
# script:       srpm.pl
# license: 	GPL v2 or later
# description:  show full information about srpm packet
# authors:      Vladimir Lettiev aka crux <crux@altlinux.ru> (c) 2004
#		Andrew Avramenko <liks@altlinux.ru> (c) 2007

use strict;
use CGI qw/param/;
use DBI;
use FindBin qw($Bin);
use lib "$Bin/adm";
use altbase qw(:DEFAULT $repocopSupport);

#> My constants
my ($select_Q,@row,@ary,$row,$sth,$dbh,$rpmprefix);
my @sections = ("docs","changelog","bugs","allbugs","spec","get","patches","getpatch",
		"getsource","gear","sources","repocop");
my $text = "";

my $cgi = new CGI;
my $alt = new altbase;

#> Load html fragments
my $lang = $cgi->param('lang');
if (! $alt->init(module => "inform", lang => $lang, script => "srpm.pl") ){
	print $altbase::header . $altbase::errStr;
	exit;
}

my ($repo,$srpm,$section,$extra) = altbase::newStyle(transPath(\$ENV{"PATH_INFO"},"w","w",\@sections,"d"));
my $show = param('show');

unless(defined($srpm)){
	print $altbase::header;
	print (($show eq "title")?"unknown":"error [stats.pl:38]");
	exit;
}

#> Connection to MySQL
$dbh = DBI->connect($altbase::dsn,$altbase::dbUser,$altbase::dbPass);

my $findSrpm_Q = "SELECT count(name) FROM srpm WHERE name=? AND repo='$repo'";

$sth = $dbh->prepare($findSrpm_Q);
$sth->execute($srpm);
$row = $sth->fetchrow_array();
$sth->finish();
if ($row == 0){
	print $altbase::header;
	print (($show eq "title")?"unknown":"error [stats.pl:52]");
	print " name=$srpm";
	exit;
} elsif ($show eq "title"){
	print $altbase::header;
	print $srpm;
	exit;
}

my $selSrpmInfo_Q = "SELECT s.version, s.rel, g.rpmgroup, m.packager, s.buildtime, \
	s.summary, s.description, s.packager, m.type, s.copyright, \
	UNIX_TIMESTAMP(NOW())-UNIX_TIMESTAMP(s.buildtime), \
	s.size, s.url, g.rpmgroup FROM srpm as s, maintainers as m, groups as g \
	WHERE m.mail=s.packager AND g.id=s.rpmgroup AND s.name=? AND repo='$repo'";

$sth = $dbh->prepare($selSrpmInfo_Q);
$sth->execute($srpm);
if (@row = $sth->fetchrow_array()){
	tagConv(\$row[6]);
	tagConv(\$row[12]);

	if (defined($section)){

		# SPEC section
		if ($section eq "spec"){

			my $SelSpec_Q = "SELECT spec FROM spec WHERE rpm=? AND repo='$repo'";
			$sth = $dbh->prepare($SelSpec_Q);
			$sth->execute($srpm);
			$row = $sth->fetchrow_array();
			specColorer(\$row);
			$text .= $alt->parser(
				HTML => "spec",
				vars => {
					"lang" => $lang,
					"text" => $row,
					"name" => $srpm,
					"path" => $altbase::path
				},
				iflist => {}
			);

		# PATCH section
		} elsif ($section eq "patches"){
			my $name = undef;
			if (defined($extra)) {
				my $patchText;
				$sth = $dbh->prepare("SELECT name,patch FROM patches WHERE srpm=? AND num=? AND repo='$repo'");
				$sth->execute($srpm,$extra);
				($name,$patchText) = $sth->fetchrow_array();
				foreach my $line (split(/\n/,$patchText)) {
					chomp($line);
					$line =~ s/&/&amp;/g;
					$line =~ s/>/&gt;/g;
					$line =~ s/</&lt;/g;
					$line =~ s/"/&quot;/g;
					$line =~ s/^(diff.+)$/<pre style="background-color:#aabccd">$1<\/pre>/;
					$line =~ s/^(\+\+\+|\-\-\-)(.+)$/<pre style="color:green">$1$2<\/pre>/;
					$line =~ s/^(\@\@.+)$/<pre style="color:gray">$1<\/pre>/;
					$line =~ s/^(\+.*)$/<pre style="color:brown">$1<\/pre>/;
					$line =~ s/^(\-.*)$/<pre style="color:blue">$1<\/pre>/;
					unless ($line =~ /^\<pre/) {
						$line = "<pre>".$line."</pre>";
					}
					$text .= $line;
				}
			} else {
				$sth = $dbh->prepare("SELECT num,name,LENGTH(patch),patch FROM patches WHERE srpm=? AND repo='$repo' ORDER by num");
				$sth->execute($srpm);
				while (@ary = $sth->fetchrow_array()) {
					$text .= $alt->parser(
						HTML => "onePatch",
						vars => {
							"lang" => $lang,
							"path" => $altbase::path,
							"srpm" => $srpm,
							"num" => $ary[0],
							"name" => $ary[1],
							"size" => &sizeOf($ary[2]),
							"repo" => $repo
						},
						iflist=>{
							 "goodpatch" => ($ary[3] eq "Sorry, cannot display patch, because patch size is bigger than 128K.")?0:1,
							 "ispatch" => 1,
							 "issource" => 0
						 	}
					);
				}
			}

			$text = $alt->parser(
				HTML => "patchList",
				vars => {
					"name" => $name, 
					"content" => $text,
					"srpm" => $srpm,
					"num" => $extra,
					"repo" => $repo
				},
				iflist => {
					"patch"=>(defined($extra))?1:0,
					"goodpatch"=>((! defined($extra)) || ($text =~ "Sorry, cannot display patch, because patch size is bigger than 128K."))?0:1
				}
			);

		# SOURCES section
		} elsif ($section eq "sources"){
			my $name = undef;
			if (defined($extra)) {
				my $sourceText;
				$sth=$dbh->prepare("SELECT name,source FROM sources WHERE srpm=? AND num=? AND repo=?");
				$sth->execute($srpm,$extra,$repo);
				($name,$sourceText) = $sth->fetchrow_array();
				$text .= "<pre>" . altbase::tagClean($sourceText) . "</pre>";
			}
			else {
				$sth = $dbh->prepare("SELECT num,name,LENGTH(source),source FROM sources WHERE srpm=? AND repo=? ORDER by num");
				$sth->execute($srpm,$repo);
				while (@ary = $sth->fetchrow_array()) {
					$text .= $alt->parser(
						HTML => "onePatch",
						vars => {
						  "lang" => $lang,
						  "path" => $altbase::path,
						  "srpm" => $srpm,
						  "num"  => $ary[0],
						  "name" => $ary[1],
						  "size" => &sizeOf($ary[2]),
						  "repo" => $repo
					  },
					  iflist=>{
						  "goodsource" => ($ary[3] eq "Source is too large.\n")?0:1,
						  "ispatch" => 0,
						  "issource" => 1
					 	  }
					);
				}
			
			$text = $alt->parser(
				HTML => "patchList",
				vars => {
				  "name" => $name, 
				  "content" => $text,
				  "srpm" => $srpm,
				  "num" => $extra,
				  "repo" => $repo
			  },
			  iflist => {
				  "source"=>(defined($extra))?1:0
			  }
			  );
		        }
		# DOWNLOAD the patch
		} elsif ($section eq "getpatch"){
		    getSource("patch");
		
		# DOWNLOAD the source
		} elsif ($section eq "getsource"){
		    getSource("source");
		
		# Gear repositories section
		} elsif ($section eq "gear"){
			my $packager = undef;
			$sth = $dbh->prepare("SELECT maintainers.packager,gitrepos.lastmodified,maintainers.mail FROM gitrepos,maintainers WHERE gitrepos.srpm=? AND maintainers.mail=gitrepos.packager ORDER by gitrepos.lastmodified DESC");
			$sth->execute($srpm);
			while (@ary = $sth->fetchrow_array()) {
			    $text .= $alt->parser(
				HTML => "oneGear",
				vars => {
					"gitpath" => $altbase::gitpath,
					"srpm" => $srpm,
					"packager" => $ary[0],
					"mail"=> $ary[2],
					"date" => $ary[1]
				},
				iflist=>{}
			    );
			}
			
			$text = $alt->parser(
				HTML => "gearList",
				vars => {
				    "content" => $text
				},
				iflist => {}
			)

		# CHANGELOG section
		} elsif ($section eq "changelog"){

			$sth = $dbh->prepare("SELECT name, time, content FROM changelogs WHERE rpm=?  AND repo='$repo' ORDER BY time DESC");
			$sth->execute($srpm);
			while(@ary = $sth->fetchrow_array()){
				$ary[0] =~ s/\@/ at /;
				$text .= $alt->parser(
					HTML => "oneChange",
					vars => {
						"time" => MakeDate($ary[1]),
						"name" => tagClean($ary[0]),
						"text" => changeFormat($ary[2])
					},
					iflist => {}
				);
			}

		# LOGS section
		} elsif ($section eq "logs"){


		# DOWNLOAD rpms section
		} elsif ($section eq "get") {

			my ($mirror,$urlsList, $caseList, $optionList, $rpmList, $arch);
			my $counter = 0;
			my $mainMirrorUrl = "";
			
			$sth = $dbh->prepare("SELECT mirror FROM mirrors ORDER BY 1 ASC");
			$sth->execute();
			while ($mirror = $sth->fetchrow_array()){

				if ($mirror =~ /altlinux\.org/ ) {
					$mainMirrorUrl = $mirror;
				}
				
				$optionList .= $alt->parser(
					HTML => "optionList",
					vars => {
						"mNum" => $counter,
						"mirrorDomain" => ( $mirror =~ m|://(.+?)/| )[0],
					},
					iflist => {
						"default" => ($mirror =~ /altlinux\.org/)?1:0,
					}
				);

				$caseList .= $alt->parser(
					HTML => "caseList",
					vars => {
						"mNum" => $counter++,
						"mirrorUrl" => $mirror
					},
					iflist => {}
				);

			}

			$counter = 0;	# reset counter
			# prepare SRPM entry
			$urlsList = $alt->parser( HTML => "urlListSRPM",
				vars => { "i" => $counter, "arch" => "src", "name" => $srpm,
					  "version" => $row[0], "release" => $row[1], "rpmprefix" => $altbase::repoinfo{$repo}{"srpm"} },
				iflist => { $repo => 1 }
			);
			$rpmList = $alt->parser ( HTML => "rpmListSRPM",
				vars => { "mainMirrorUrl" => $mainMirrorUrl, "i" => $counter++, "name" => $srpm,
					  "arch"=>"src", "version" => $row[0], "release" => $row[1],
					  "size" => &sizeOf($row[11]), "rpmprefix" => $altbase::repoinfo{$repo}{"srpm"} },
				iflist => { $repo => 1 }
			);
			# adding RPMS entries
			my $selRPMs_Q = "SELECT name,version,rel,arch,size FROM rpm WHERE srpm=? AND repo='$repo'"
					. "AND arch=? ORDER BY name ASC";
			$sth = $dbh->prepare($selRPMs_Q);
			my @arch=("i386","i486","i586","noarch","x86_64");
			foreach $arch (@arch) {
			    $sth->execute($srpm,$arch);
    			    $rpmList .= "<tr><td><br><b>$arch:</b></td></tr>" if ($sth -> fetchrow_array());
			    # FIXME: It is not a good idea to execute the same function 2 times! But now I have no ideas :(
			    $sth->execute($srpm,$arch);
			    $rpmprefix = $altbase::repoinfo{$repo}{"rpm"};
			    $rpmprefix =~ s/<arch>/$arch/;
			    while (@ary = $sth->fetchrow_array()){
				$urlsList .= $alt->parser(
					HTML => "urlListRPM",
					vars => {
						"i" => $counter,
						"arch" => $ary[3],
						"name" => $ary[0],
						"version" => $ary[1],
						"release" => $ary[2]
					},
					iflist => {
						$repo => 1,
						"src" => 0
					}
				); 
				$rpmList .= $alt->parser(
					HTML => "rpmListRPM",
					vars => {
						"mainMirrorUrl" => $mainMirrorUrl,
						"i" => $counter++,
						"arch" => $ary[3],
						"name" => $ary[0],
						"version" => $ary[1],
						"release" => $ary[2],
						"size" => &sizeOf($ary[4]),
						"rpmprefix" => $rpmprefix
					},
					iflist => {
					    $repo => 1,
					}
				);
			    }
			}
			$text = $alt->parser(
				HTML => "getSource",
				vars => {
					"urlsList" => $urlsList,
					"numRpm" => $counter,
					"caseList" => $caseList,
					"optionList" => $optionList,
					"rpmList" => $rpmList
				},
				iflist => {}
			);

		# DOCS section. FIXME: Not finished yet
		} elsif ($section eq "docs"){

		# BUGS section
		} elsif ($section eq "bugs" || $section eq "allbugs"){

			my $i = 0;
			my %countBugs = ();
			my $statusFilter = "";
			my @statusFilter = ();
			
			foreach ('UNCONFIRMED','NEW','ASSIGNED','REOPENED') {
				push @statusFilter,"bugs.status='$_'";
			}
			$statusFilter = " AND (" . join(' OR ',@statusFilter) . ")";

			my $selBugs_Q = "SELECT bugs.bug_id, bugs.severety, bugs.priority, bugs.reporter, \
			    		 bugs.status, bugs.resolution, bugs.description FROM bugs ";
			if (($repo eq "orphaned") || ($repo eq "obsolete")) {
			    $selBugs_Q .= "LEFT JOIN srpm ON srpm.name=bugs.rpm WHERE srpm.name=? AND srpm.repo='$repo' ";
			}
			else {
			    $selBugs_Q .= "LEFT JOIN rpm ON rpm.name=bugs.rpm WHERE rpm.srpm=? AND rpm.repo='Sisyphus' AND rpm.arch != 'x86_64'";
			}
			    $selBugs_Q .= $statusFilter if ($section eq "bugs");
			    $selBugs_Q .= " ORDER BY resolution ASC, status ASC, severety DESC, priority ASC, bug_id DESC";
			    
			$sth = $dbh->prepare($selBugs_Q);
			$sth->execute($srpm);
			while (@ary = $sth->fetchrow_array){
				$i++;
				$ary[3] =~ s/\@/ at /;
				$text .= $alt->parser(
					HTML => "bug",
					vars => {
						"id" => $ary[0],
                                		"severety" => $ary[1],
						"priority" => $ary[2],
						"reporter" => tagClean($ary[3]),
						"status" => $ary[4],
						"resolution" => ($ary[5])?$ary[5]:"&nbsp;",
						"description" => tagClean($ary[6])
					},
					iflist => {}
				);
			}
			
			my $selBugsCount_Q;
			if (($repo eq "orphaned") || ($repo eq "obsolete")) {
			    $selBugsCount_Q = "SELECT count(bugs.bug_id) FROM bugs LEFT JOIN srpm ON " .
			    "srpm.name=bugs.rpm WHERE srpm.name=? AND srpm.repo='$repo'";
			}
			else {
    			    $selBugsCount_Q = "SELECT count(bugs.bug_id) FROM bugs LEFT JOIN rpm ON " .
			    "rpm.name=bugs.rpm WHERE rpm.srpm=? AND rpm.repo='Sisyphus' AND rpm.arch != 'x86_64'";
			}
			$selBugsCount_Q .= $statusFilter if ($section eq "allbugs");

			$sth = $dbh->prepare($selBugsCount_Q);
			$sth->execute($srpm);
			$countBugs{"get"} = $sth->fetchrow_array();
			$sth->finish();

			if ($section eq "bugs") {
				$countBugs{"allBugsCount"}  = $countBugs{"get"};
				$countBugs{"openBugsCount"} = $i;
			} else {
				$countBugs{"allBugsCount"}  = $i;
				$countBugs{"openBugsCount"} = $countBugs{"get"};
			}

			$text = $alt->parser(
				HTML => "bugslist",
				vars => {
					"lang" => $lang,
					"content" => $text,
					"path" => $altbase::path,
					"name" => $srpm,
					"repo" => $repo,
					"allBugsCount" => $countBugs{"allBugsCount"},
					"openBugsCount" => $countBugs{"openBugsCount"},
					"closedBugsCount" => $countBugs{"allBugsCount"} - $countBugs{"openBugsCount"}
				},
				iflist => {
					"bugs" => ($countBugs{"allBugsCount"} == 0)?0:1,
					"open" => ($section eq "bugs" && $countBugs{"allBugsCount"} >0 )?1:0,
					"all" => ($section eq "allbugs" && $countBugs{"allBugsCount"} >0 )?1:0
				}
			);

		# REPOCOP section
		} elsif ($section eq "repocop"){
			my $selRepocop_Q = "SELECT CONCAT(repocop.name,'-',repocop.version,'-',repocop.rel,'.',repocop.arch), repocop.status, repocop.testname, repocop.message FROM repocop where repocop.srcname=? AND repocop.status<>'skip'";
			$selRepocop_Q .= " ORDER BY repocop.status DESC, repocop.version DESC, repocop.rel DESC, repocop.arch DESC";
			
			my $countRepocop=0;
			$sth = $dbh->prepare($selRepocop_Q) || die $dbh->errstr;
			$sth->execute($srpm) || die $dbh->errstr;
			while (@ary = $sth->fetchrow_array){
				$countRepocop++;
				$text .= $alt->parser(
					HTML => "repocopmsg",
					vars => {
						"id" => $ary[0],
                                		"status" => $ary[1],
						"testname" => $ary[2],
						"message" => $ary[3],
					},
					iflist => {}
				);
			}
			

			$text = $alt->parser(
				HTML => "repocoplist",
				vars => {
					"content" => $text,
				},
				iflist => {
					"repocopmsgs" => $countRepocop,
					"repocopnomsgs" => (not $countRepocop),
					"repocopsupport" => $repocopSupport,
				}
			);

		# This is never happen...
		} else {
			print $altbase::header;
			print "section=$section<br>";
			print "internal error [srpm.pl:94]";
			exit;
		}
	# Section not specified. So its a main page
	} else {
		$text = $alt->parser(
			HTML => "package",
			vars => {
				"lang" => $lang,
				"path" => $altbase::path,
				"name" => $srpm,
				"version" => $row[0],
				"release" => $row[1],
				"packager" => ( $row[3] =~ /^(.+?)\</ )[0],
				"date" => MakeDateTime($row[4]),
				"url" => $row[12],
				"sub" => sprintf("%.1f",$row[10]/(3600*24*7)),
				"summary" => $row[5],
				"description" => $row[6],
				"mail" => $row[7],
				"size" => &sizeOf($row[11]),
				"license" => $row[9],
				"mainlist" => &allMan($srpm),
				"rpmlist" => &allRpm($srpm),
				"acllist" => &allAcl($srpm)
			},
			iflist => {
				"team" => ($row[8] eq "team")?0:1,
				"url" => ($row[12] eq "")?0:1
			}
		);
	}

	$text = $alt->parser(
		HTML => "title",
		vars => {
			"lang" => $lang,
			"content" => $text,
			"navigator" => &getNavigator($row[7],$row[13],$srpm),
			"group" => &localeGrp($row[13]),
			"name" => $srpm,
			"path" => $altbase::path,
			"repo" => $repo
		},
		iflist => {
			"main" => (! defined($section))?1:0,
			"spec" => ($section eq "spec")?1:0,
			"changelog" => ($section eq "changelog")?1:0,
#			"docs" => ($section eq "docs")?1:0,
			"bugs" => ($section eq "bugs" || $section eq "allbugs")?1:0,
			"get" => ($section eq "get")?1:0,
			"patches" => ($section eq "patches")?1:0,
			"gear" => ($section eq "gear")?1:0,
			"sources" => ($section eq "sources")?1:0,
			'repocop' => ($section eq "repocop")?1:0,
			'repocopsupport' => $repocopSupport,
		}
	);
} else {
	exit;
}

$dbh->disconnect();
print $altbase::header . $text;

sub allMan {
	my $srpm = shift;
	my $alt = new altbase;
	my (@allman, $row, $mail, $text, $sth, @row);
	my $selAllMan_Q = "SELECT name FROM changelogs WHERE rpm=? AND repo='$repo' ORDER BY time DESC";

	$text = "";

	$sth = $dbh->prepare($selAllMan_Q);
	$sth->execute($srpm);

	while ($row = $sth->fetchrow_array()){
		($mail) = ($row =~ /<\s*([\w|\-]+)\s*[\@|at]/);
		next if (isInArray($mail,\@allman));
		push(@allman,$mail);
	}

	my $selManInfo_Q = "SELECT packager, type FROM maintainers WHERE mail=?";
	$sth = $dbh->prepare($selManInfo_Q);

	foreach my $man (@allman){
		$sth->execute($man);

		if (@row = $sth->fetchrow_array()){
			$text .= $alt->parser(
				HTML => "packager",
				vars => {
					"lang" => $lang,
					"path" => $altbase::path,
					"mail" => $man,
					"packager" => ( $row[0] =~ /^(.+?)\</ )[0]
				},
				iflist => {
					"team" => ($row[1] eq "team")?0:1
				}
			);
		}
	}

	$sth->finish();	#FIXME: Does it needed?
	return $text;
}

sub allAcl {
	my $srpm = shift;
	my ($sth, $sth2, @row, $row, $text);
	my $alt = new altbase;
	
	my $selACLs_Q = "SELECT packager FROM acl WHERE srpm=?";
	$sth = $dbh->prepare($selACLs_Q);
	$sth->execute($srpm);

	my $selManInfo_Q = "SELECT packager, type FROM maintainers WHERE mail=?";
	$sth2 = $dbh->prepare($selManInfo_Q);
	while ($row = $sth->fetchrow_array()){
		$row =~ s/^\@//;
		$sth2->execute($row);
		while (@row = $sth2->fetchrow_array()){
		    $text .= $alt->parser(
			HTML => "packager",
			vars => {
				"lang" => $lang,
			        "path" => $altbase::path,
			        "mail" => $row,
			        "packager" => ( $row[0] =~ /^(.+?)\</ )[0]
			},
			iflist => {
			        "team" => ($row[1] eq "team")?0:1
			}
		   )
		}
	}
	$sth->finish();
	$sth2->finish();
	return $text;
}

sub allRpm {
	my $srpm = shift;
	my ($sth,$row,$text);
	my $alt = new altbase;
	my $selRPMS_Q = "SELECT name from rpm WHERE srpm=? AND repo='$repo' AND arch!='x86_64' ORDER BY name ASC";

	$sth = $dbh->prepare($selRPMS_Q);
	$sth->execute($srpm);
	while ( $row = $sth->fetchrow_array() ){
		$text .= $alt->parser(
			HTML => "rpm",
			vars => { "name" => $row },
			iflist => {}
		);
	}

	return $text;
}

sub sizeOf {
	my $size = shift;

	if ($size < 1024*10) {
		return $size." b";
	} elsif ($size < 1024*1024*2) {
		return sprintf("%.2f",$size/1024)." Kb";
	} else {
		return sprintf("%.2f",$size/(1024*1024))." Mb";
	}
}

#FIXME: End of good formatting text
sub getNavigator {
    my ($main, $grp, $srpm) = @_;
    my $alt = new altbase;
    my $text = "";
    my @list = ();
    my %nav=();
    my ($sth,$row,$i);

    my $selMainSrpms_Q = "SELECT name FROM srpm WHERE packager=?  AND repo='$repo' ORDER BY name ASC";
    $sth = $dbh->prepare($selMainSrpms_Q);
    $sth->execute($main);
    while ($row = $sth->fetchrow_array){
        push(@list,$row);
    }
    if (@list) {
        for ($i=0; $i<=$#list; $i++){
            if ($list[$i] eq $srpm) {
                last;
            }
        }
        if ($i > 0) {
            $nav{"prevmain"} = $list[$i-1];
        }
        if ($i < $#list) {
            $nav{"nextmain"} = $list[$i+1];
        }
    }

    @list = ();
    my $selGrpSrpms_Q = "SELECT srpm.name FROM srpm,groups WHERE groups.id=srpm.rpmgroup AND groups.rpmgroup=?  AND srpm.repo='$repo'  ORDER BY name ASC";
    $sth = $dbh->prepare($selGrpSrpms_Q);
    $sth->execute($grp);
    while ($row = $sth->fetchrow_array){
        push(@list,$row);
    }
    if (@list) {
        for ($i=0; $i<=$#list; $i++){
            if ($list[$i] eq $srpm) {
                last;
            }
        }
        if ($i > 0) {
            $nav{"prevgrp"} = $list[$i-1];
        }
        if ($i < $#list) {
            $nav{"nextgrp"} = $list[$i+1];
        }
    }

    $text = $alt->parser(
                 HTML => "navigator",
                 vars => {
                     "lang" => $lang,
                     "path" => $altbase::path,
                     "prevmain" => $nav{"prevmain"},
                     "nextmain" => $nav{"nextmain"},
                     "prevgrp" => $nav{"prevgrp"},
                     "nextgrp" => $nav{"nextgrp"}
                 },
                 iflist => {
                     "prevmain" => ($nav{"prevmain"})?1:0,
                     "nextmain" => ($nav{"nextmain"})?1:0,
                     "prevgrp" => ($nav{"prevgrp"})?1:0,
                     "nextgrp" => ($nav{"nextgrp"})?1:0
                 }
    );

    return $text;
}

# Download sources or patches
sub getSource {
	my $type = shift;
	my $name = shift;
	if (defined($extra)) {
	    my $sourceText;
	    my $table = undef;
	   
	    $table = "patches" if ($type eq "patch"); 
	    $table = "sources" if ($type eq "source");
	
	    $sth = $dbh->prepare("SELECT name,$type FROM $table WHERE srpm='$srpm' AND repo='$repo' AND num='$extra'");
	    $sth->execute();
	    ($name,$sourceText) = $sth->fetchrow_array();
	    
	    print "Content-type: text/plain\n";
	    print "Content-disposition: attachment; filename=$name\n\n";
	    print "$sourceText";
	    exit 0;
	}
}
