#!/usr/bin/perl
#
# script:      stats.pl
# description: show statictics
# author:      Vladimir Lettiev aka crux <crux@altlinux.ru> (c) 2004
#

use strict;
use CGI qw/param/;
use DBI;
use FindBin qw($Bin);
use lib "$Bin/adm";
use altbase;

#> My constants
my ($select_Q,$name,$dbh,$sth,$sth2,@row,@ary,$ary);
my $text = "";

my $cgi = new CGI;
my $alt = new altbase;

#> Load html fragments
my $lang = $cgi->param('lang');
if (! $alt->init(module => "inform", lang => $lang, script => "stats.pl") ){
	print $altbase::header . $altbase::errStr;
	exit;
}

my $type = $cgi->param('type');
my $transId = $cgi->param('tid');

if (!defined($transId) || $transId =~ /\D/) {
	$transId = undef;
}

$dbh = DBI->connect($altbase::dsn,$altbase::dbUser,$altbase::dbPass);

if ($type eq "leader") {

	$select_Q = "SELECT count(srpm.name), maintainers.packager, srpm.packager \
	FROM srpm, maintainers WHERE maintainers.mail=srpm.packager AND \
	maintainers.type='man' AND srpm.repo='Sisyphus' GROUP BY srpm.packager ORDER BY 1 DESC LIMIT 15";

	$sth = $dbh->prepare($select_Q);
	$sth->execute();
	while (@row = $sth->fetchrow_array){
		$row[1] =~ s/\s*<.+$//;
		$text .= $alt->parser(
			HTML => "leader",
			vars => {
				"lang" => $lang,
				"path" => $altbase::path,
				"name" => $row[2],
				"packager" => tagClean($row[1]),
				"count" => $row[0]
			},
			iflist => {}
		);
	}

} elsif ($type eq "activity") {

	$select_Q = "SELECT rpm, count(time) FROM changelogs WHERE " .
	"time>(DATE_SUB(NOW(),INTERVAL 6 MONTH)) AND repo='Sisyphus' GROUP BY rpm ORDER BY 2 DESC, 1 ASC LIMIT 15";

	$sth = $dbh->prepare($select_Q);
	$sth->execute();
	while (@row = $sth->fetchrow_array){
		$text .= $alt->parser(
			HTML => "activity",
			vars => {
				"lang" => $lang,
				"path" => $altbase::path,
				"name" => $row[0],
				"display" => (length($row[0]) > 24)? substr($row[0],0,20) . "&hellip;" : $row[0],
				"activity" => $row[1]
			},
			iflist => {}
		);

	}

} elsif ($type eq "lastrpm" || ($type eq "transfer" && defined($transId))) {

	my ($tid, $time, $b_size, $s_size);

	if ($type eq "transfer") {

		$sth = $dbh->prepare("SELECT tid, time, b_size, s_size FROM transfer WHERE tid=?");
		$sth->execute($transId);
		($tid, $time, $b_size, $s_size) = $sth->fetchrow_array();
		unless($tid) {
			$dbh->disconnect();
			print $altbase::header;
			print "transfer not found";
			exit;
		}

	} else {
	
		$sth = $dbh->prepare("SELECT tid,time,b_size,s_size FROM transfer ORDER BY tid DESC LIMIT 1");
		$sth->execute();
		($tid, $time, $b_size, $s_size) = $sth->fetchrow_array();
		
	}
	my ($newContent, $updContent, $delContent, $newQuant, $updQuant, $delQuant) = ();

	$sth = $dbh->prepare(
		"SELECT " .
			"t.srpm, "         . # 0
			"t.from_version, " . # 1
			"t.to_version, "   . # 2
			"t.status, "	   . # 3
			"s.version, "      . # 4
			"s.rel, "          . # 5
			"g.rpmgroup, "     . # 6
			"m.packager, "     . # 7
			"s.buildtime, "    . # 8
			"s.summary, "      . # 9
			"c.content , "     . #10
			"s.packager, "     . #11
			"m.type, "         . #12
			"c.packager "      . #13
		"FROM translist as t, srpm as s, maintainers as m, changelogs as c, groups as g " .
		"WHERE t.srpm=s.name AND m.mail=s.packager AND g.id=s.rpmgroup AND c.version=t.to_version " .
		"AND c.rpm=t.srpm AND t.tid=? AND t.repo='Sisyphus' " .
		"AND s.repo='Sisyphus' AND c.repo='Sisyphus' " .
		"GROUP BY t.srpm ORDER by t.status, t.srpm"
	);
	$sth->execute($tid);
	while (@row = $sth->fetchrow_array()) {
		# NMU check
		if ($row[11] ne $row[13] && $row[11] ne "qa-robot") {
			$sth2 = $dbh->prepare("SELECT packager FROM maintainers WHERE mail=?");
			$sth2->execute($row[13]);
			$row[14] = $sth2->fetchrow_array();
		} else {
			$row[14] = $row[7];
		}

		# Changelog check
		if ($row[3] eq "updated") {
			$row[10] = "";
			$sth2 = $dbh->prepare("SELECT time FROM changelogs WHERE version=? AND rpm=? AND repo='Sisyphus' " .
				"ORDER by time DESC LIMIT 1");
			$sth2->execute($row[2],$row[0]);
			$ary = $sth2->fetchrow_array();
				
			$sth2 = $dbh->prepare("SELECT c.name, c.content, c.time FROM translist as t, changelogs as c " .
				"WHERE t.tid=? AND t.srpm=? AND c.rpm=t.srpm AND c.time = ? AND c.repo='Sisyphus' ".
				"AND t.repo='Sisyphus' ORDER BY c.time DESC"
			);
			$sth2->execute($tid,$row[0],$ary);
			my $i = 0;
			while (@ary = $sth2->fetchrow_array()) {
				$ary[0] =~ s/\@/ at /;
				$row[10] .= $alt->parser(
					HTML=>"changelog",
					vars=>{
						"name" => $ary[0],
						"content" => $ary[1]
					},
					iflist => {
						"first" => ($i++ == 0)
					}
				);
			}
		} else {
			&tagConv(\$row[10]);
		}
		# Cut email part of packager/builder name
		$row[14] =~ s/\s*<.+$//;
		$row[7]  =~ s/\s*<.+$//;

		#FIXME: packages must be here
		my $temp = $alt->parser(
			HTML => "package",
			vars => {
				"lang" => $lang,
				"path" => $altbase::path,
				"name" => $row[0],
				"version" => $row[4],
				"release" => $row[5],
				"group" => &localeGrp($row[6]),
				"mail" => $row[11],
				"packager" => tagClean($row[7]),
				"builder" => tagClean($row[14]),
				"buildermail" => tagClean($row[13]),
				"date" => MakeDateTime($row[8]),
				"summary" => tagClean($row[9]),
				"changes" => $row[10],
			},
			iflist => {
				"team" => ($row[12] eq "team"),
				"NMU" => ($row[11] ne $row[13]),
				"NMUteam" => ($row[11] ne $row[13] && $row[12] ne "team"),
				"qarobot" => ($row[13] eq "qa-robot"),
				"new" => ($row[3] eq "new")
			}
		);
		if ($row[3] eq "new") {
			$newContent .= $temp;
			$newQuant++;
		} else {
			$updContent .= $temp;
			$updQuant++;
		}
	}

	$sth = $dbh->prepare("SELECT srpm, from_version FROM translist WHERE tid=? AND status='deleted' AND repo='Sisyphus'");
	$sth->execute($tid);
	while (@row = $sth->fetchrow_array()){
		$delQuant++;
		$delContent .= $alt->parser(
			HTML=>"orphPackage",
			vars=>{
				"name" => $row[0],
				"vr" => $row[1]
			},
			iflist=>{}
		);
	}

	$text = $alt->parser(
		HTML => "transfer",
		vars => {
			"date" => &MakeDate($time),
			"newQuant" => $newQuant,
			"updQuant" => $updQuant,
			"delQuant" => $delQuant,
			"s_size" => &sizeOf($s_size),
			"b_size" => &sizeOf($b_size),
			"sum" => &sizeOf($s_size+$b_size),
			"newContent" => $newContent,
			"updContent" => $updContent,
			"delContent" => $delContent
		},
		iflist => {
			"haveNew" => ($newQuant),
			"haveUpd" => ($updQuant),
			"haveDel" => ($delQuant),
			"transfer" => ($s_size)
		}
	);
	
# Show list of transfers
} elsif ($type eq "transfer") {
	my %key = ( 'new'=>0, 'updated'=>0, 'deleted'=>0);
	$sth = $dbh->prepare("SELECT tid, time, b_size+s_size from transfer ORDER BY tid DESC");
	$sth->execute();
	while(@row = $sth->fetchrow_array()){
		foreach my $status (keys %key) {
			$sth2 = $dbh->prepare("SELECT count(*) from translist WHERE tid=? AND status=? AND repo='Sisyphus'");
			$sth2->execute($row[0],$status);
			$key{$status} = $sth2->fetchrow_array();
		}

		$text .= $alt->parser(
			HTML=>"transList",
			vars=>{
				"lang"=>$lang,
				"tid"=>$row[0],
				"time"=>&MakeDate($row[1]),
				"size"=>&sizeOf($row[2]),
				%key
			},
			iflist=>{}
		);
	}
}

$dbh->disconnect();

print $altbase::header;
print $text;

sub sizeOf {
        my $size = shift;

        if ($size < 1024*10) {
                return $size." b";
        } elsif ($size < 1024*1024*2) {
                return sprintf("%.2f",$size/1024)." Kb";
        } else {
                return sprintf("%.2f",$size/(1024*1024))." Mb";
        }
}
